/*
 * Copyright 2011 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ssg.tools.jsonxml;

import com.ssg.tools.jsonxml.impl.ParserContext;
import com.ssg.tools.jsonxml.utils.Formats;
import java.io.IOException;
import java.io.Reader;
import java.util.logging.Logger;

/**
 *
 * @author ssg
 */
public abstract class Parser {

    public static final int READER_EOF = '\uffff';
    static Logger log = Logger.getLogger(Parser.class.getName());
    static char[] WHITE_SPACE_CHARS = new char[]{' ', '\t', '\n', '\r'};

    public abstract Object parse(Reader r) throws IOException;

    public abstract Object parse(Reader r, Formats formatters) throws IOException;

    public abstract Object parse(Reader r, ParserContext context) throws IOException;

    public static char nextNonSpaceChar(Reader r) throws IOException {
        char ch = ' ';
        while (ch != READER_EOF && isWhiteSpace(ch)) {
            ch = readChar(r);
        }
        return ch;
    }

    public static char readChar(Reader r) throws IOException {
        return (char) r.read();
    }

    public static boolean isWhiteSpace(char ch) {
        if (ch != READER_EOF) {
            for (int i = 0; i < WHITE_SPACE_CHARS.length; i++) {
                if (ch == WHITE_SPACE_CHARS[i]) {
                    return true;
                }
            }
        }
        return false;
    }
}
