/*
 * Copyright 2011 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ssg.tools.jsonxml;

import com.ssg.tools.jsonxml.impl.DumpFormat;
import com.ssg.tools.jsonxml.impl.JSONFormat;
import com.ssg.tools.jsonxml.impl.JSONParser;
import com.ssg.tools.jsonxml.impl.StructureFormat;
import com.ssg.tools.jsonxml.impl.XMLFormat;
import com.ssg.tools.jsonxml.impl.XMLParser;
import com.ssg.tools.jsonxml.utils.Formats;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.io.Writer;
import java.net.URL;
import java.util.Locale;

/**
 * 
 * @author ssg
 */
public class converter {

    public static void help() {
        System.out.println("Convertion tool: reads JSON/XML text and writes to STDOUT re-formatted one.");
        System.out.println(" converter [-format=JSON|XML] [-indent=true|false] [-locale=locale] [-help] <input file or URL>");
        System.out.println("   By default reads STDIN and writes to STDOUT in indented JSON format.");
    }

    public static void main(String[] args) throws Exception {
//        LogManager lm = LogManager.getLogManager();
//        lm.readConfiguration(Thread.currentThread().getContextClassLoader().getResourceAsStream("logger.test.properties"));
//        Logger log = Logger.getLogger(converter.class.getName());
        boolean indent = true;
        String format = "JSON";
        Locale locale = Locale.getDefault();
        String path = null;

        if (args.length == 0) {
            help();
            return;
        }
        for (String arg : args) {
            if (arg.startsWith("-h") || arg.startsWith("-H") || arg.startsWith("-?")) {
                help();
                return;
            } else if (arg.startsWith("-format=")) {
                format = arg.substring("-format=".length());
                if (format.equalsIgnoreCase("json")) {
                    format = "JSON";
                } else if (format.equalsIgnoreCase("XML")) {
                    format = "XML";
                } else if (format.equals("Dump")) {
                    format = "Dump";
                } else {
                    help();
                    System.out.println("  wrong parameter: " + arg + ". Error: unknown output format.");
                }

            } else if (arg.startsWith("-indent=")) {
                try {
                    indent = Boolean.parseBoolean(arg.substring("-indent=".length()));
                } catch (Throwable th) {
                    help();
                    System.out.println("  wrong parameter: " + arg + ". Error: " + th);
                }
            } else if (arg.startsWith("-locale=")) {
                try {
                    locale = new Locale(arg.substring("-locale=".length()));
                } catch (Throwable th) {
                    help();
                    System.out.println("  wrong parameter: " + arg + ". Error: " + th);
                }
            } else if (arg.startsWith("-")) {
                help();
                System.out.println("  unknown parameter: " + arg + ".");
            } else {
                path = arg;
            }
        }

        if (path == null) {
            help();
            System.out.println("  ERROR: input file name or URL is missing.");
        }

        Formats formats = new Formats(locale, null);
        Object obj = null;
        Throwable xmlErr = null;
        Throwable jsonErr = null;
        try {
            Reader r = new InputStreamReader((path.contains("//")) ? (new URL(path)).openStream() : new FileInputStream(new File(path)));
            obj = (new XMLParser()).parse(r, formats);
        } catch (Throwable th) {
            xmlErr = th;
        }
        if (obj == null) {
            try {
                Reader r = new InputStreamReader((path.contains("//")) ? (new URL(path)).openStream() : new FileInputStream(new File(path)));
                obj = (new JSONParser()).parse(r, new Formats(locale, null));
            } catch (Throwable th) {
                jsonErr = th;
            }
        }
        if (obj == null) {
            help();
            System.out.println("  ERROR: failed to parse as XML : " + xmlErr);
            System.out.println("  ERROR: failed to parse as JSON: " + jsonErr);
            return;
        }

        StructureFormat sf = null;
        if (format.equals("JSON")) {
            sf = new JSONFormat(indent);
        } else if (format.equals("XML")) {
            sf = new XMLFormat(indent);
        } else if (format.equals("Dump")) {
            sf = new DumpFormat(indent);
        }

        Writer writer = new OutputStreamWriter(System.out);
        Formatter.toFormattedString(writer, obj, "", sf, false);
        writer.close();
    }

    public static void putStringToFile(File file, String s) throws IOException {
        if (file != null && file.exists()) {
            file.delete();
        }
        Writer writer = new FileWriter(file);
        writer.write(s);
        writer.close();
    }

    /**
     * Tries to load data as XML or JSON. On success returns true otherwise - false.
     */
    public static Object loadData(Reader r) {
        if (r != null) {
            // try as XML
            try {
                Object obj = ((new XMLParser()).parse(r));
                if (obj != null) {
                    return obj;
                }
                return true;
            } catch (Throwable th) {
                th.printStackTrace();
            }
            // if no success try as JSON
            try {
                Object obj = ((new JSONParser()).parse(r));
                if (obj != null) {
                    return obj;
                }
                return true;
            } catch (Throwable th) {
                th.printStackTrace();
            }
        }
        return null;
    }
}
