/*
 * Copyright 2011 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ssg.tools.jsonxml.examples;

import com.ssg.tools.jsonxml.impl.ReflectiveXMLParser;
import com.ssg.tools.jsonxml.Formatter;
import com.ssg.tools.jsonxml.impl.DumpFormat;
import com.ssg.tools.jsonxml.impl.JSONParser;
import com.ssg.tools.jsonxml.impl.JSONParser.JSONParserContext;
import com.ssg.tools.jsonxml.impl.ParserContext;
import com.ssg.tools.jsonxml.impl.XMLParser.XMLParserContext;
import com.ssg.tools.jsonxml.utils.ReflectiveBuilder;
import java.io.StringReader;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 *
 * @author ssg
 */
public class ReflectionSample {

    public enum ROLE {

        employee, employer, candidate, partner, customer, other
    }

    public static class Contact {

        private String name;
        private String alias;
        private String phone;
        private Address address;
        private ROLE role = ROLE.employer;

        /**
         * @return the name
         */
        public String getName() {
            return name;
        }

        /**
         * @param name the name to set
         */
        public void setName(String name) {
            this.name = name;
        }

        /**
         * @return the alias
         */
        public String getAlias() {
            return alias;
        }

        /**
         * @param alias the alias to set
         */
        public void setAlias(String alias) {
            this.alias = alias;
        }

        /**
         * @return the phone
         */
        public String getPhone() {
            return phone;
        }

        /**
         * @param phone the phone to set
         */
        public void setPhone(String phone) {
            this.phone = phone;
        }

        /**
         * @return the address
         */
        public Address getAddress() {
            return address;
        }

        /**
         * @param address the address to set
         */
        public void setAddress(Address address) {
            this.address = address;
        }

        /**
         * @return the role
         */
        public ROLE getRole() {
            return role;
        }

        /**
         * @param role the role to set
         */
        public void setRole(ROLE role) {
            this.role = role;
        }
    }

    public static class Company extends Contact {

        private List<Contact> contacts = new ArrayList<Contact>();
        private Address address2;
        private Address address3;
        private Date created = new Date();
        private Map<Contact, Contact> relations = new LinkedHashMap<Contact, Contact>();

        public Company() {
            setRole(ROLE.employee);
        }

        /**
         * @return the contacts
         */
        public List<Contact> getContacts() {
            return contacts;
        }

        /**
         * @param contacts the contacts to set
         */
        public void setContacts(List<Contact> contacts) {
            this.contacts = contacts;
        }

        /**
         * @return the address2
         */
        public Address getAddress2() {
            return address2;
        }

        /**
         * @param address2 the address2 to set
         */
        public void setAddress2(Address address2) {
            this.address2 = address2;
        }

        /**
         * @return the address3
         */
        public Address getAddress3() {
            return address3;
        }

        /**
         * @param address3 the address3 to set
         */
        public void setAddress3(Address address3) {
            this.address3 = address3;
        }

        /**
         * @return the created
         */
        public Date getCreated() {
            return created;
        }

        /**
         * @param created the created to set
         */
        public void setCreated(Date created) {
            this.created = created;
        }

        /**
         * @return the relations
         */
        public Map<Contact, Contact> getRelations() {
            return relations;
        }

        /**
         * @param relations the relations to set
         */
        public void setRelations(Map<Contact, Contact> relations) {
            this.relations = relations;
        }
    }

    public static class GeoLocation {

        private double longitude;
        private double latitude;
        private BigDecimal height;

        /**
         * @return the longitude
         */
        public double getLongitude() {
            return longitude;
        }

        /**
         * @param longitude the longitude to set
         */
        public void setLongitude(double longitude) {
            this.longitude = longitude;
        }

        /**
         * @return the latitude
         */
        public double getLatitude() {
            return latitude;
        }

        /**
         * @param latitude the latitude to set
         */
        public void setLatitude(double latitude) {
            this.latitude = latitude;
        }

        /**
         * @return the height
         */
        public BigDecimal getHeight() {
            return height;
        }

        /**
         * @param height the height to set
         */
        public void setHeight(BigDecimal height) {
            this.height = height;
        }
    }

    public static class Address {

        private String street;
        private String city;
        private String country;
        private GeoLocation location;

        /**
         * @return the street
         */
        public String getStreet() {
            return street;
        }

        /**
         * @param street the street to set
         */
        public void setStreet(String street) {
            this.street = street;
        }

        /**
         * @return the city
         */
        public String getCity() {
            return city;
        }

        /**
         * @param city the city to set
         */
        public void setCity(String city) {
            this.city = city;
        }

        /**
         * @return the country
         */
        public String getCountry() {
            return country;
        }

        /**
         * @param country the country to set
         */
        public void setCountry(String country) {
            this.country = country;
        }

        /**
         * @return the location
         */
        public GeoLocation getLocation() {
            return location;
        }

        /**
         * @param location the location to set
         */
        public void setLocation(GeoLocation location) {
            this.location = location;
        }
    }

    public static Company getSampleCompany() {
        Company c = new Company();
        c.setCreated(new Date());
        c.setName("Company A");
        c.setAlias("CA");
        Address a1 = new Address();
        a1.setCountry("R");
        a1.setCity("C");
        a1.setStreet("Asdfg 23-4, 12");
        GeoLocation location = new GeoLocation();
        location.setLatitude(30.004);
        location.setLongitude(60.2);
        location.setHeight(new BigDecimal("34563657645757.76"));
        a1.setLocation(location);
        c.setAddress(a1);

        Contact c1 = new Contact();
        c1.setName("c1");
        Contact c2 = new Contact();
        c2.setName("c2");

        c.getRelations().put(c1, c2);
        c.getRelations().put(c2, c1);
        return c;
    }

    public static void main(String[] args) throws Exception {
        ReflectiveXMLParser xrp = new ReflectiveXMLParser();
        ParserContext xpc = new XMLParserContext();
        ReflectiveBuilder rb = xpc.getReflectiveBuilder();
        ParserContext jpc = new JSONParserContext();
        jpc.setReflectiveBuilder(rb);
        //rb.registerClass(Address.class);
        //rb.registerClass(Company.class);
        //rb.registerClass(Contact.class);
        //rb.mapClass("Address", Address.class);
        rb.mapClass("Company", Company.class);

        Company c = getSampleCompany();

        String json1 = Formatter.toJSONString("Company", c, true);
        String xml1 = Formatter.toXMLString("Company", c, true);
        //System.out.println(json1);
        System.out.println(xml1);

        //xpc.setFormats(new Formats(Locale.US, null));
        Object obj = (new JSONParser()).parse(new StringReader(xml1), jpc);
        String dump1 = Formatter.toFormattedString(obj, new DumpFormat(true));
        obj = xrp.parse(new StringReader(xml1), xpc);
        String xml2 = Formatter.toXMLString(obj, true);
        String dump2 = Formatter.toFormattedString(obj, new DumpFormat(true));
        System.out.println(xml2);
        System.out.println(dump2);
        System.out.println(dump1);
        // compare original formatted and formatted-parsed-formatted versions.
        String s1 = Formatter.toJSONString("Company", c, true);
        String s2 = Formatter.toJSONString(obj, true);
        System.out.println("Original===converted = " + (s1.equals(s2)));
        System.out.println(s1);
        System.out.println(s2);
    }
}
