/*
 * Copyright 2011 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ssg.tools.jsonxml.examples;

import com.ssg.tools.jsonxml.Formatter;
import com.ssg.tools.jsonxml.impl.DumpFormat;
import com.ssg.tools.jsonxml.impl.JSONFormat;
import com.ssg.tools.jsonxml.impl.JSONParser;
import com.ssg.tools.jsonxml.impl.JSONParser.JSONParserContext;
import com.ssg.tools.jsonxml.impl.ParserContext;
import com.ssg.tools.jsonxml.impl.StructureFormat;
import com.ssg.tools.jsonxml.impl.XMLFormatForJSON1;
import com.ssg.tools.jsonxml.impl.XMLParser;
import com.ssg.tools.jsonxml.utils.Formats;
import java.io.IOException;
import java.io.StringReader;
import java.text.DateFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 *
 * @author ssg
 */
public class SimpleCases {

    public static void main(String[] args) throws IOException, ParseException {
        // prepare to test different date/time formats: date, time, date time
        Formats formats = new Formats();
        DateFormat ddf = formats.getDateFormat("yyyy-MM-dd");
        DateFormat tdf = formats.getDateFormat("HH:mm");
        DateFormat dtdf = formats.getDateFormat("yyyy-MM-dd HH:mm");

        // simple structure
        Map root = new HashMap();
        root.put("A", "a");
        List list = new ArrayList();
        list.add("B");
        list.add(new Date());
        list.add(tdf.parse("14:02"));
        list.add(ddf.parse("2011-04-23"));
        list.add(3.1415926);
        root.put("items", list);

        // show object structure
        System.out.println("As JSON:");
        System.out.println(Formatter.toJSONString(root, false));
        System.out.println("As XML :");
        System.out.println(Formatter.toXMLString(root, false));
        System.out.println("As XML with wrapping element:");
        System.out.println(Formatter.toXMLString("sample", root, false));
        System.out.println("As XML1:");
        System.out.println(Formatter.toXMLString(root, new XMLFormatForJSON1(false)));
        System.out.println("As dump:");
        System.out.println(Formatter.toFormattedString(root, new DumpFormat(false)));

        // parse from JSON
        String s0 = Formatter.toJSONString(root, false);
        Object o1 = (new JSONParser()).parse(new StringReader(s0));
        String s1 = Formatter.toJSONString(o1, false);
        System.out.println("Compare parsed to orihinal in JSON: equals = " + s0.equals(s1) + ";");
        System.out.println("    " + s0);
        System.out.println("    " + s1);
        // parse from XML
        s0 = Formatter.toXMLString("sample", root, false);
        o1 = (new XMLParser()).parse(new StringReader(s0));
        s1 = Formatter.toXMLString(o1, false);
        System.out.println("Compare parsed to orihinal in XML : equals = " + s0.equals(s1) + ";");
        System.out.println("    " + s0);
        System.out.println("    " + s1);
        // dump context for format/parse for JSON
        StructureFormat sf = new JSONFormat();
        ParserContext pctx = new JSONParserContext();
        s0 = Formatter.toFormattedString(root, sf);
        o1 = ((new JSONParser())).parse(new StringReader(s0), pctx);
        System.out.println("Dump structure format and parser context for JSON:");
        System.out.println("    " + Formatter.toFormattedString(sf, new DumpFormat(true)));
        System.out.println("    " + Formatter.toFormattedString(pctx, new DumpFormat(true)));
    }
}
