/*
 * Copyright 2011 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ssg.tools.jsonxml.impl;

import com.ssg.tools.jsonxml.utils.StructureProcessingException;
import com.ssg.tools.jsonxml.utils.Path;

/**
 * Implements reflective XML parser.
 * Uses list of supported classes with optional mappings.
 * At least 1 mapping is expected, preferably for root object(s).
 *
 * @author ssg
 */
public class ReflectiveXMLParser extends XMLParser {

    public ReflectiveXMLParser() {
    }

    @Override
    public void postProcessReflectiveObject(XMLParserContext context, String name, Object object, int index) throws StructureProcessingException {
        if (context.getReflectiveBuilder() != null) {
            Object parent = (context.getStack().size() > 1) ? context.getStack().get(context.getStack().size() - 2) : null;
            parent = context.getStack().peek();
            context.getReflectiveBuilder().reflectiveBuild(parent, object, Path.createFor(context.getPath(), name, null), context.getFormats(), context.getObjectsRegistry());
        }
    }
}
