/*
 * Copyright 2011 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ssg.tools.jsonxml.impl;

import com.ssg.tools.jsonxml.utils.STRUCTURE_FORMAT_ELEMENT;
import com.ssg.tools.jsonxml.utils.PRIMITIVE_TYPE;
import com.ssg.tools.jsonxml.utils.Utilities.Pair;
import java.io.IOException;
import java.io.Writer;

/**
 *
 * @author ssg
 */
public class StructureWriter extends Writer {

    Writer _base = null;
    private StructureFormat _format;

    public StructureWriter(Writer base) {
        _base = base;
    }

    public StructureWriter(Writer base, StructureFormat format) {
        _base = base;
        _format = format;
    }

    @Override
    public void write(char[] cbuf, int off, int len) throws IOException {
        if (_base != null) {
            _base.write(cbuf, off, len);
        }
    }

    @Override
    public void flush() throws IOException {
        if (_base != null) {
            _base.flush();
        }
    }

    @Override
    public void close() throws IOException {
        if (_base != null) {
            _base.close();
        }
    }

    /**
     * "contextual" writes used in Formatter.
     * 
     * @param element
     * @param s
     * @throws IOException 
     */
    public void write(STRUCTURE_FORMAT_ELEMENT element, String s) throws IOException {
        write(s);
    }

    /**
     * "contextual" writes used in Formatter.
     * 
     * @param element
     * @param s
     * @throws IOException 
     */
    public void write(STRUCTURE_FORMAT_ELEMENT element, Pair<PRIMITIVE_TYPE, String> s) throws IOException {
        write(s.getB());
    }

    /**
     * @return the _format
     */
    public StructureFormat getFormat() {
        return _format;
    }

    /**
     * @param format the _format to set
     */
    public void setFormat(StructureFormat format) {
        this._format = format;
    }
}
