/*
 * Copyright 2011 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *
 * JSONXML is library used to parse/format tree-like object structures in most popular text formats: XML and JSON.
 * It may be considered as "alternative" serializer but it is not though may be adapted.
 *
 * For parsing it accepts "java.io.Reader" and return java object.
 * For formatting it accepts java object and returns String or may return result to "java.io.Writer".
 *
 * Object is generally structure that contains Map and/or List elements.
 * Map is ordered set of named items. List is set of unnamed items.
 * Reflection may be used to convert objects into set of maps/lists and vice versa.
 *
 * JSON parser is implemented explicitly.
 * XML parser is based on SAX parser and applies only certain rules for result.
 * 
 * Simple use:
 * Use Formatter class's static methods toXMLString, toJSONString or, generally, to FormattedString 
 * to convert object to string.
 * 
 * Use instance of XMLParser or JSONParser, method "parse", to convert string to Map/List java objects.
 * Use ReflectiveXMLParser or ReflectiveJSONParser, method "parse", to convert string to desired objects.
 * 
 * NOTE: Reflective parsing needs registered name to class mappings to restore objects. 
 *       On any error in converting Map/List structure to expected class it will stop conversion and
 *       return same result as normal parser though it may contain other successfully restored objects.
 * 
 * Library is designed to allow various entry points for specific decisions depending on end use needs.
 *  1. Formats - formats are used to enable locale-specific parsing/formatting of numbers and dates.
 *  2. ReflectiveBuilder - enables reflection. Default implementation uses getters/setters only.
 *  3. ObjectsRegistry - used to keep track of parsed or formatted objects and allow 
 *     resolvable references in formatted (text) form.
 *  4. StructureFormat - generic formatter context object providing all info needed in process
 * 
 * Sub-package "utils" holds classes invariant on library (mey be used independently).
 * Sub-package "impl" holds implementation of basic and default realizations of parsers and formatters.
 * Sub-package "examples" gives quick introduction to variants of use.
 * 
 * For more examples see unit tests, especially functionalTest.java.
 * 
 */
package com.ssg.tools.jsonxml;
