/*
 * Copyright 2011 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ssg.tools.jsonxml.utils;

/**
 * Structure to hold parsing error info.
 * 
 * @author ssg
 */
public class StructureErrorInfo {

    private int _line;
    private int _pos;
    private String _expected;
    private String _found;
    private String _description;

    public StructureErrorInfo(int line, int pos, String expected, String found) {
        _line = line;
        _pos = pos;
        _expected = expected;
        _found = found;
    }

    public StructureErrorInfo(String description, String expected, String found) {
        _description = description;
        _line = -1;
        _pos = -1;
        _expected = expected;
        _found = found;
    }

    @Override
    public String toString() {
        if (getDescription() != null) {
            return getDescription() + ((getExpected() != null || getFound() != null) ? ", expected: " + getExpected() + ", found=" + getFound() : "");
        } else {
            return "line: " + getLine() + ", pos: " + getPos() + ", expected: " + getExpected() + ", found=" + getFound();
        }
    }

    /**
     * @return the _line
     */
    public int getLine() {
        return _line;
    }

    /**
     * @param line the _line to set
     */
    public void setLine(int line) {
        this._line = line;
    }

    /**
     * @return the _pos
     */
    public int getPos() {
        return _pos;
    }

    /**
     * @param pos the _pos to set
     */
    public void setPos(int pos) {
        this._pos = pos;
    }

    /**
     * @return the _expected
     */
    public String getExpected() {
        return _expected;
    }

    /**
     * @param expected the _expected to set
     */
    public void setExpected(String expected) {
        this._expected = expected;
    }

    /**
     * @return the _found
     */
    public String getFound() {
        return _found;
    }

    /**
     * @param found the _found to set
     */
    public void setFound(String found) {
        this._found = found;
    }

    /**
     * @return the _description
     */
    public String getDescription() {
        return _description;
    }

    /**
     * @param description the _description to set
     */
    public void setDescription(String description) {
        this._description = description;
    }
}
