/*
 * Copyright 2011 ssg.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.ssg.tools.jsonxml.utils;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Stack;

/**
 * Base structure processing context used in parsers and formatters.
 * Provides basic set of "tools" for use: 
 *  - tree navigation stack
 *  - current path
 *  - objects registry
 *  - formats (locale/time zone specific)
 *  - reflection support (reflective builder)
 *  - errors registration
 * 
 * @author ssg
 */
public class StructureProcessingContext {

    private Stack _stack = new Stack() {

        @Override
        public synchronized Object pop() {
            Object object = super.pop();
            //System.out.println("StructureFormat.Stack.pop  " + size() + " -> " + object);
            return object;
        }

        @Override
        public Object push(Object item) {
            Object object = super.push(item);
            //System.out.println("StructureFormat.Stack.push " + size() + " -> " + object);
            return object;
        }
    };
    private Path _path = new Path(null) {

        @Override
        public void add(String path, String index) {
            super.add(path, index);
            //System.out.println("StructureFormat.Path.add " + path + " -> " + toString());
        }

        @Override
        public String remove() {
            String removed = super.remove();
            //System.out.println("StructureFormat.Path.remove " + removed + " -> " + toString());
            return removed;
        }
    };
//    private Stack _stack = new Stack();
//    private Path _path = new Path("/", -1);
    private Formats _formats = new Formats();
    private ObjectsRegistry _objectsRegistry = new ObjectsRegistry();
    private List<StructureErrorInfo> _errors = new ArrayList<StructureErrorInfo>();
    private ReflectiveBuilder _reflectiveBuilder = new ReflectiveBuilder();

    public StructureProcessingContext() {
    }

    public StructureProcessingContext(Locale locale) {
        _formats = new Formats(locale, null);
    }

    /**
     * @return the _stack
     */
    public Stack getStack() {
        return _stack;
    }

    /**
     * @return the _path
     */
    public Path getPath() {
        return _path;
    }

    /**
     * @return the _formats
     */
    public Formats getFormats() {
        return _formats;
    }

    /**
     * @param formats the _formats to set
     */
    public void setFormats(Formats formats) {
        this._formats = formats;
    }

    /**
     * @param stack the _stack to set
     */
    protected void setStack(Stack stack) {
        this._stack = stack;
    }

    /**
     * @param path the _path to set
     */
    protected void setPath(Path path) {
        this._path = path;
    }

    /**
     * @return the _objectsRegistry
     */
    public ObjectsRegistry getObjectsRegistry() {
        return _objectsRegistry;
    }

    /**
     * @param objectsRegistry the _objectsRegistry to set
     */
    public void setObjectsRegistry(ObjectsRegistry objectsRegistry) {
        this._objectsRegistry = objectsRegistry;
    }

    /**
     * @return the _errors
     */
    public List<StructureErrorInfo> getErrors() {
        return _errors;
    }

    public void addError(StructureErrorInfo error) {
        if (error != null) {
            _errors.add(error);
        }
    }

    /**
     * @return the _reflectiveBuilder
     */
    public ReflectiveBuilder getReflectiveBuilder() {
        return _reflectiveBuilder;
    }

    /**
     * @param reflectiveBuilder the _reflectiveBuilder to set
     */
    public void setReflectiveBuilder(ReflectiveBuilder reflectiveBuilder) {
        this._reflectiveBuilder = reflectiveBuilder;
    }
}
